import json
import logging
from typing import Optional
from accqsure.exceptions import SpecificationError


class Inspections(object):
    def __init__(self, accqsure):

        self.accqsure = accqsure

    async def get(self, id_, **kwargs):

        resp = await self.accqsure._query(f"/inspection/{id_}", "GET", kwargs)
        return Inspection(self.accqsure, **resp)

    async def list(self, type_, limit=50, start_key=None, **kwargs):

        resp = await self.accqsure._query(
            "/inspection",
            "GET",
            {"type": type_, "limit": limit, "start_key": start_key, **kwargs},
        )
        inspections = [
            Inspection(self.accqsure, **inspection)
            for inspection in resp.get("results")
        ]
        return inspections, resp.get("last_key")

    async def create(
        self,
        type_,
        name,
        document_type_id,
        manifests,
        draft=None,
        documents=None,
        **kwargs,
    ):

        data = dict(
            name=name,
            type=type_,
            document_type_id=document_type_id,
            manifests=manifests,
            draft=draft,
            documents=documents,
            **kwargs,
        )
        payload = {k: v for k, v in data.items() if v is not None}
        logging.info("Creating Inspection %s", name)

        resp = await self.accqsure._query("/inspection", "POST", None, payload)
        inspection = Inspection(self.accqsure, **resp)
        logging.info("Created Inspection %s with id %s", name, inspection.id)

        return inspection

    async def remove(self, id_, **kwargs):

        await self.accqsure._query(f"/inspection/{id_}", "DELETE", {**kwargs})


class Inspection:
    def __init__(self, accqsure, **kwargs):
        self.accqsure = accqsure
        self._entity = kwargs
        self._id = self._entity.get("entity_id")
        self._name = self._entity.get("name")
        self._type = self._entity.get("type")
        self._status = self._entity.get("status")
        self._doc_content_id = self._entity.get("doc_content_id")
        self._content_id = self._entity.get("content_id")

    @property
    def id(self) -> str:
        return self._id

    @property
    def type(self) -> str:
        return self._type

    @property
    def status(self) -> str:
        return self._status

    @property
    def name(self) -> str:
        return self._name

    def __str__(self):
        return json.dumps({k: v for k, v in self._entity.items()})

    def __repr__(self):
        return f"Inspection( accqsure , **{self._entity.__repr__()})"

    def __bool__(self):
        return bool(self._id)

    async def remove(self):

        await self.accqsure._query(
            f"/inspection/{self._id}",
            "DELETE",
        )

    async def rename(self, name):

        resp = await self.accqsure._query(
            f"/inspection/{self._id}",
            "PUT",
            None,
            dict(name=name),
        )
        self.__init__(self.accqsure, **resp)
        return self

    async def refresh(self):

        resp = await self.accqsure._query(
            f"/inspection/{self.id}",
            "GET",
        )
        self.__init__(self.accqsure, **resp)
        return self

    async def _set_asset(self, path, file_name, mime_type, contents):
        return await self.accqsure._query(
            f"/inspection/{self.id}/asset/{path}",
            "PUT",
            params={"file_name": file_name},
            data=contents,
            headers={"Content-Type": mime_type},
        )

    async def get_doc_contents(self):
        if not self._doc_content_id:
            raise SpecificationError(
                "doc_content_id",
                "Document content not uploaded for inspection",
            )

        resp = await self.accqsure._query(
            f"/inspection/{self.id}/asset/{self._doc_content_id}/manifest.json",
            "GET",
        )
        return resp

    async def get_doc_content_item(self, name):
        if not self._doc_content_id:
            raise SpecificationError(
                "doc_content_id", "Document not uploaded for inspection"
            )

        return await self.accqsure._query(
            f"/inspection/{self.id}/asset/{self._doc_content_id}/{name}",
            "GET",
        )

    async def _set_doc_content_item(
        self, name, file_name, mime_type, contents
    ):
        if not self._doc_content_id:
            raise SpecificationError(
                "content_id", "Content not finalized for inspection"
            )
        return await self._set_asset(
            f"{self._doc_content_id}/{name}", file_name, mime_type, contents
        )

    async def get_contents(self):
        if not self._content_id:
            raise SpecificationError(
                "content_id", "Content not finalized for inspection"
            )

        resp = await self.accqsure._query(
            f"/inspection/{self.id}/asset/{self._content_id}/manifest.json",
            "GET",
        )
        return resp

    async def get_content_item(self, name):
        if not self._content_id:
            raise SpecificationError(
                "content_id", "Content not finalized for inspection"
            )

        return await self.accqsure._query(
            f"/inspection/{self.id}/asset/{self._content_id}/{name}",
            "GET",
        )

    async def _set_content_item(self, name, file_name, mime_type, contents):
        if not self._content_id:
            raise SpecificationError(
                "content_id", "Content not finalized for inspection"
            )
        return await self._set_asset(
            f"{self._content_id}/{name}", file_name, mime_type, contents
        )

    async def list_checks(
        self,
        document_id=None,
        manifest_id=None,
        limit=50,
        start_key=None,
        name=None,
        **kwargs,
    ):

        resp = await self.accqsure._query(
            f"/inspection/{self.id}/check",
            "GET",
            {
                "document_id": document_id,
                "manifest_id": manifest_id,
                "limit": limit,
                "start_key": start_key,
                "name": name,
                **kwargs,
            },
        )
        checks = [
            InspectionCheck(self.accqsure, self, **check)
            for check in resp.get("results")
        ]
        return checks, resp.get("last_key")


class InspectionCheck:
    def __init__(self, accqsure, inspection, **kwargs):
        self.accqsure = accqsure
        self._entity = kwargs
        self._inspection = inspection
        self._id = self._entity.get("entity_id")
        self._section = self._entity.get("check_section")
        self._name = self._entity.get("check_name")
        self._status = self._entity.get("status")
        self._critical = self._entity.get("critical")
        self._compliant = self._entity.get("compliant")
        self._rationale = self._entity.get("rationale")
        self._suggestion = self._entity.get("suggestion")

    @property
    def id(self) -> str:
        return self._id

    @property
    def section(self) -> str:
        return self._section

    @property
    def name(self) -> str:
        return self._name

    @property
    def status(self) -> str:
        return self._status

    @property
    def critical(self) -> bool:
        return self._critical

    @property
    def compliant(self) -> Optional[bool]:
        return self._compliant

    @property
    def rationale(self) -> Optional[str]:
        return self._rationale

    @property
    def suggestion(self) -> Optional[str]:
        return self._suggestion

    def __str__(self):
        return json.dumps({k: v for k, v in self._entity.items()})

    def __repr__(self):
        return f"InspectionCheck( accqsure , **{self._entity.__repr__()})"

    def __bool__(self):
        return bool(self._id)

    async def update(self, **kwargs):

        resp = await self.accqsure._query(
            f"/inspection/{self._inspection.id}/check/{self.id}",
            "PUT",
            None,
            dict(**kwargs),
        )
        self.__init__(self.accqsure, self._inspection, **resp)
        return self

    async def refresh(self):

        resp = await self.accqsure._query(
            f"/inspection/{self._inspection.id}/check/{self.id}",
            "GET",
        )
        self.__init__(self.accqsure, self._inspection, **resp)
        return self



import click
import os
import base64
import mimetypes
from enum import Enum
from accqsure.cli import cli, pass_config


class FILE_FORMATS(str, Enum):
    DOCX = "docx"
    TEXT = "text"
    XLSX = "xlsx"
    CSV = "csv"
    PDF = "pdf"


DOCUMENT_TYPES = {
    "application/vnd.openxmlformats-officedocument.wordprocessingml.document": FILE_FORMATS.DOCX,
    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet": FILE_FORMATS.XLSX,
    "text/plain": FILE_FORMATS.TEXT,
    "application/json": FILE_FORMATS.TEXT,
    "text/csv": FILE_FORMATS.CSV,
    "text/markdown": FILE_FORMATS.TEXT,
    "application/pdf": FILE_FORMATS.PDF,
}


@cli.group()
@pass_config
def document(config):
    """AccQsure document commands."""
    pass


@document.command()
@click.argument("document_type_id", type=click.STRING)
@pass_config
def list(config, document_type_id):
    """List documents."""
    data = [
        ["ID", "NAME", "DOC_ID"],
        [
            "-" * 80,
            "-" * 80,
            "-" * 80,
        ],
    ]

    # TODO: implement cursor function
    documents, _ = config.accqsure.run(
        config.accqsure.client.documents.list(
            document_type_id=document_type_id
        )
    )
    print(documents)
    for doc in documents:
        data.append(
            [
                doc.id,
                doc.name,
                doc.doc_id,
            ]
        )
    for row in data:
        click.echo(
            "{: >26.24} {: >40.38} {: >14.12} " "".format(*row),
            file=config.stdout,
        )


@document.command()
@click.option("--document-type", "-t", type=click.STRING, required=True)
@click.option(
    "--file",
    "-f",
    type=click.Path(
        exists=True, file_okay=True, dir_okay=False, resolve_path=True
    ),
    required=True,
)
@click.argument("name", type=click.STRING, required=True)
@click.argument("doc-id", type=click.STRING, required=True)
@pass_config
def create(config, name, doc_id, document_type_id, file):
    """
    Add a document

    """

    mime_type, _ = mimetypes.guess_type(file)

    if mime_type not in DOCUMENT_TYPES:
        raise ValueError(
            f"Invalid file type. Detected MIME type '{mime_type}' not in allowed types: {', '.join(DOCUMENT_TYPES.values())}"
        )

    file_type = DOCUMENT_TYPES[mime_type]

    with open(os.path.expanduser(file), "rb") as f:
        value = f.read()
        base64_contents = base64.b64encode(value).decode("utf-8")

    title = os.path.splitext(os.path.basename(file))[0]
    result = config.accqsure.run(
        config.accqsure.client.documents.create(
            document_type_id=document_type_id,
            name=name,
            doc_id=doc_id,
            contents=dict(
                title=title, type=file_type, base64_contents=base64_contents
            ),
        )
    )
    click.echo(result)



import json


class PlotElements(object):
    def __init__(
        self,
        accqsure,
        plot_id,
        section_id,
    ):
        self.accqsure = accqsure
        self.plot_id = plot_id
        self.section_id = section_id

    async def get(self, id_, **kwargs):

        resp = await self.accqsure._query(
            f"/plot/{self.plot_id}/section/{self.section_id}/element/{id_}",
            "GET",
            kwargs,
        )
        return PlotElement(
            self.accqsure, self.plot_id, self.section_id, **resp
        )

    async def list(self, limit=50, start_key=None, **kwargs):

        resp = await self.accqsure._query(
            f"/plot/{self.plot_id}/section/{self.section_id}/element",
            "GET",
            {"limit": limit, "start_key": start_key, **kwargs},
        )
        plots = [
            PlotElement(self.accqsure, self.plot_id, self.section_id, **plot)
            for plot in resp.get("results")
        ]
        return plots, resp.get("last_key")


class PlotElement:
    def __init__(self, accqsure, plot_id, section_id, **kwargs):
        self.accqsure = accqsure
        self.plot_id = plot_id
        self.section_id = section_id
        self._entity = kwargs
        self._id = self._entity.get("entity_id")
        self._order = self._entity.get("order")
        self._type = self._entity.get("type")
        self._status = self._entity.get("status")
        self._content = self._entity.get("content")

    @property
    def id(self) -> str:
        return self._id

    @property
    def order(self) -> int:
        return self._order

    @property
    def status(self) -> str:
        return self._status

    @property
    def content(self) -> str:
        return self._content

    def __str__(self):
        return json.dumps({k: v for k, v in self._entity.items()})

    def __repr__(self):
        return f"PlotElement( accqsure , **{self._entity.__repr__()})"

    def __bool__(self):
        return bool(self._id)

    async def refresh(self):

        resp = await self.accqsure._query(
            f"/plot/{self.plot_id}/section/{self.section_id}/element/{self.id}",
            "GET",
        )
        self.__init__(self.accqsure, **resp)
        return self



import click
from accqsure.cli import cli, pass_config


@cli.group()
@pass_config
def text(config):
    """AccQsure text commands."""
    pass


@text.command()
@click.argument("prompt", type=click.STRING)
@pass_config
def generate(config, prompt):
    """
    Generate a Text Completion Response

    """

    result = config.accqsure.run(
        config.accqsure.client.text.generate([dict(role="user", content=prompt)])
    )
    click.echo(result)


@text.command()
@click.argument("text", type=click.STRING)
@pass_config
def tokenize(config, text):
    """
    Count the Tokens in a Text String

    """

    result = config.accqsure.run(config.accqsure.client.text.tokenize(text))
    click.echo(result.get("tokens"))


@text.command()
@click.argument("text", type=click.STRING)
@pass_config
def vectorize(config, text):
    """
    Convert a Text String into an Embedding Vector

    """

    result = config.accqsure.run(config.accqsure.client.text.vectorize(text))
    click.echo(result.get("vector"))
